#version 440 core

/* Input (interpolated values from vertex shader) */
in vec2 UV;
in vec3 position_worldspace;
in vec3 normal_cameraspace;
in vec3 cameraDirection_cameraspace;
in vec3 lightDirection_cameraspace;

/* Output */
layout(location = 0) out vec4 color;
layout(location = 2) out vec4 brightColor;

/* Constants for whole mesh */
uniform sampler2D textureSampler;
const vec3 lightPosition_worldspace = vec3(0, 0, 0);

/* Other variables */
vec3 lightColor;
float lightBrightness;
vec3 materialColor_diffuse;
vec3 materialColor_ambient;
vec3 materialColor_specular;
float dist;
vec3 normal;
vec3 direction;
float cosTheta;
vec3 cameraVector;
vec3 reflectionVector;
float cosAlpha;

void main() {

	/* Light settings */
	lightColor =				vec3(1, 1, 0.7);
	lightBrightness =			800000.0f;

	/* Material settings */
	materialColor_diffuse =		texture(textureSampler, UV).rgb;
	materialColor_ambient =		vec3(0.2, 0.2, 0.2) * materialColor_diffuse;
	materialColor_specular =	vec3(0.4, 0.4, 0.4);

	/* Distance to light source */
	dist =						length(lightPosition_worldspace - position_worldspace);

	/* Normal of fragment in camera space */
	normal =					normalize(normal_cameraspace);

	/* Direction to light */
	direction =					normalize(lightDirection_cameraspace);

	/* Cosine of angle between normal and direction of light - clamped above 0 */
	cosTheta =					clamp(dot(normal, direction), 0, 1);

	/* Vector towards camera */
	cameraVector =				normalize(cameraDirection_cameraspace);

	/* Direction of triangle reflecting light */
	reflectionVector =			reflect(-direction, normal);

	/* Cosine of angle between camera vector and reflection vector - clamped to 0 */
	cosAlpha =					clamp(dot(cameraVector, reflectionVector), 0, 1);

	vec3 tempColor;

	if (cosTheta > 0) {
		tempColor =		materialColor_ambient +
								materialColor_diffuse * lightColor * lightBrightness * cosTheta / (dist*dist) +
								materialColor_specular * lightColor * lightBrightness * pow(cosAlpha, 5) / (dist*dist);
	}

	else {
		tempColor =				materialColor_ambient +
								materialColor_diffuse * lightColor * lightBrightness * cosTheta / (dist*dist) +
								0 * materialColor_specular * lightColor * lightBrightness * pow(cosAlpha, 5) / (dist*dist);
	}

	color = vec4(tempColor, 1.0f);

	float brightness = dot(color.rgb, vec3(0.2126, 0.7152, 0.0722));
	if(brightness > 1.8) {
		brightColor = vec4(color.rgb, 1.0f);
	}
}