#version 440 core

void applyBasic();
void applyHDR();
void applyMotionBlur();
void applyDOF();
void applyBloom();
//void applyLensflare();
//vec3 distortTexture(
//	in sampler2D tex,
//	in vec2 textureCoords,
//	in vec2 direction,
//	in vec3 distortion
//);

/* Input */
in vec2 UV;

/* Output - Interpolated for each fragment*/
out vec4 color;
vec4 DOFcolor;
vec4 motionBlurcolor;
vec4 HDRcolor;
vec4 bloomColor;

/* Constants for whole mesh */
uniform sampler2D fboTexture;
uniform sampler2D fboTexture2;
uniform sampler2D bloomBlur;
uniform sampler2D fboDepthTexture;
uniform mat4 inverseVP;			// inverse view-projection matrix
uniform mat4 previousVP;		// previous view-projection matrix

uniform float width;
uniform float height;

const int samples = 8;
//const float texelSizeWidth = 1.0f/width;
//const float texelSizeHeight = 1.0f/height;
const vec4 DOFparameter = vec4(0.0, 0.5, -0.9, 1.0);
//uniform sampler1D lensColor;
//uniform float dispersal = 0.25f;
//uniform float haloWidth = 1.0f;
//uniform float distortion = 1.0f;
//uniform vec4 scale = vec4(1.0f);
//uniform vec4 bias = vec4(0.0f);

void main() {
	//applyBasic();
	applyHDR();
	applyBloom();
	applyDOF();
	applyMotionBlur();
	color = 0.85 * DOFcolor + 0.03 * motionBlurcolor + 0.04 * HDRcolor + 0.08 * bloomColor;
	color.a = 1.0f;
	//applyLensflare();
}

/*void applyBasic() {
	color = texture(fboTexture, UV);
}*/

void applyDOF() {

	float texelSizeWidth = 1.0f/width;
	float texelSizeHeight = 1.0f/height;

	/* Get depth buffer value at this pixel */
	float cameraDepth = texture(fboDepthTexture, UV).x;
	vec4 blurredColor = 0.25 * (texture(fboTexture2, UV+vec2(texelSizeWidth, texelSizeHeight)) +
								texture(fboTexture2, UV-vec2(texelSizeWidth, texelSizeHeight)) +
								texture(fboTexture2, UV+vec2(texelSizeWidth, -texelSizeHeight)) +
								texture(fboTexture2, UV+vec2(-texelSizeWidth, texelSizeHeight)));
	float factor;

	if (cameraDepth == -1.0) {
		DOFcolor = blurredColor;
	}

	else {
		if (cameraDepth < DOFparameter.x || cameraDepth > DOFparameter.z) {
			DOFcolor = blurredColor;
		}

		else {
			if (cameraDepth > DOFparameter.y) {
				factor = (DOFparameter.z - cameraDepth) / (DOFparameter.z - DOFparameter.y);
			}

			else {
				factor = (cameraDepth - DOFparameter.x) / (DOFparameter.y - DOFparameter.x);
			}
			DOFcolor = factor * texture(fboTexture, UV) + (1.0 - factor) * blurredColor;
		}
	}	
}

void applyMotionBlur() {
	/* Extract per-pixel world space positions of objects that were rendered to depth texture */
	/* Get depth buffer value at this pixel */
	float zOverW = texture(fboDepthTexture, UV).x;
	/* H is the viewport position at this pixel in range -1, 1 */
	vec4 H = vec4(UV.x * 2 - 1, (1 - UV.y) * 2 - 1, zOverW, 1);
	/* Transform by view-projection inverse */
	vec4 D = inverseVP * H;
	/* Divide by w to get world position */
	vec4 worldPos = D / D.w;

	/* Compute per-pixel velocity vectors that determine the blur direction */
	/* Current viewport position */
	vec4 currentPos = H;
	/* Use world position, transform by previous view-projection matrix */
	vec4 previousPos = previousVP * worldPos;
	/* Convert to nonhomogenous points in range -1, 1 by dividing by w */
	previousPos /= previousPos.w;
	/* Use this frame's and last frame's position to compute pixel velocity */
	vec2 velocity = (currentPos - previousPos).xy / (samples/2.0);

	/* Blurring */
	motionBlurcolor = texture(fboTexture, UV);
	vec2 UV2 = UV;
	UV2 -= velocity;
	vec4 currentColor;
	for (int i = 1; i < samples; i++) {
		UV2 -= velocity;
		currentColor = texture(fboTexture, UV2);
		motionBlurcolor += currentColor;
	}

	motionBlurcolor /= float(samples);
}

void applyHDR() {
	const float gamma = 1.2;
	const float exposure = 0.2;

	vec3 hdrColor = texture(fboTexture, UV).rgb;

	/* tone mapping including exposure */
	vec3 toneMappedColor = vec3(1.0) - exp(-hdrColor * exposure);
	/* gamme correction */
	toneMappedColor = pow(toneMappedColor, vec3(1.0 / gamma));

	HDRcolor = vec4(toneMappedColor, 1.0f);
}

void applyBloom() {
	const float gamma = 2.2;
    vec3 outColor = texture(fboTexture, UV).rgb;      
    vec3 bloom = texture(bloomBlur, UV).rgb;
    outColor += bloom; // additive blending
    /* tone mapping */
    vec3 result = vec3(1.0) - exp(-outColor * 0.2);
    /* gamma correction */
    result = pow(result, vec3(1.0 / gamma));
    bloomColor = vec4(result, 1.0f);
}



//void applyLensflare() {
	
//	/*vec4 tempResult = max(vec4(0.0), texture(fboTexture, UV) + bias) * scale;
//	color.x *= tempResult.x;
//	color.y *= tempResult.y;
//	color.z *= tempResult.z;*/

//	vec2 texcoord = -UV + vec2(1.0);	// separate lens flare from source image
//	vec2 texelSize = 1.0 / vec2(textureSize(fboTexture, 0));
	
//	vec2 ghostVec = (vec2(0.5) - texcoord) * dispersal;
//	vec2 haloVec = normalize(ghostVec) * haloWidth;
	
//	vec3 distortion = vec3(-texelSize.x * distortion, 0.0, texelSize.x * distortion);

///* Ghosts */
//	vec3 result = vec3(0.0);
//	for (int i = 0; i < samples; ++i) {
//		vec2 offset = fract(texcoord + ghostVec * float(i));
		
//		float weight = length(vec2(0.5) - offset) / length(vec2(0.5));
//		weight = pow(1.0 - weight, 10.0);
	
//		result += distortTexture(
//			fboTexture,
//			offset,
//			normalize(ghostVec),
//			distortion
//		) * weight;
//	}
	
//	result *= texture(lensColor, length(vec2(0.5) - texcoord) / length(vec2(0.5)));

///* Halo */
//	float weight = length(vec2(0.5) - fract(texcoord + haloVec)) / length(vec2(0.5));
//	weight = pow(1.0 - weight, 10.0);
//	result += distortTexture(
//		fboTexture,
//		fract(texcoord + haloVec),
//		normalize(ghostVec),
//		distortion
//	) * weight;
	
//	color = result;
//}

//vec3 distortTexture(
//	in sampler2D tex,
//	in vec2 textureCoords,
//	in vec2 direction,
//	in vec3 distortion
//) {
//	return vec3(
//		texture(tex, textureCoords + direction * distortion.r).r,
//		texture(tex, textureCoords + direction * distortion.g).g,
//		texture(tex, textureCoords + direction * distortion.b).b
//		// 1.0f in case of vec4
//	);
//}